#ifndef COMMON_CK
#define COMMON_CK

#include "WHRN.h"
#include "rnkiss.h"

using namespace MyRNKISS;

namespace cmmn {



	double normal_0_to_z( double z );

	double Cum_NormalKC( double z );

	double PFPsi( double x, double g, double a, double b );


	class Rec {
		public:
			double ixrec;
			int    irrec;
	};

	class Psi {
		public:
			double *** ptlambda;		//	prior at trial t,	lambda = (g, a, b)
			double *** ptlambda0;		//	prior used in the final analysis
			double ** ptrx;				//	Pt(r|x)
			double ***** ptl_xr;		//	Pt(lambda|x,r)
			double ** H;				//	Ht(x,r)
			double * EH;				//	Ht(x) = E(Ht(x,r))
			Rec * rec;
			int xt1, rt1, ipos;

			int Ng,					//	= 31;	No. of values of guessing probabilities
				Na,					//	= 31;	No. of values of Alpha
				Nb,					//	= 71;	No. of values of Beta
				NXValue,			//	= 10;	No. of x vlues...0, 1, 2, 3
				NBlocks,			//	= 35;	No. of blocks
				NSubTrials,			//	= 3;	No. of subtrials;
				NTrial;				//	= 40;  No. of trials per series
			double  La,				//	= 1.0e-6;	The lowest value of Alpha
					Lb,				//	= 1.0e-6;	The lowest value of Beta
					BkGrnd;			//	= 120.0;		Back ground stimulus

			Psi(): Ng(31), Na(51), Nb(71), NXValue(15), NBlocks(35), NSubTrials(3), La(1.0e-6), Lb(1.0e-6), BkGrnd(150.0) {
				NTrial = NBlocks * NSubTrials;
				ptlambda = new double**[Ng];
				ptlambda0 = new double**[Ng];
				for (int i = 0; i < Ng; i++){
					ptlambda[i] = new double*[Na];
					ptlambda0[i] = new double*[Na];
					for (int j = 0; j < Na; j++){
						ptlambda[i][j] = new double[Nb];
						ptlambda0[i][j] = new double[Nb];
					}
				}
				ptrx = new double*[2];
				for (int i = 0; i < 2; i++)
					ptrx[i] = new double[NXValue];
				ptl_xr = new double****[Ng];
				for (int i = 0; i < Ng; i++){
					ptl_xr[i] = new double***[Na];
					for (int j = 0; j < Na; j++){
						ptl_xr[i][j] = new double**[Nb];
						for (int k = 0; k < Nb; k++){
							ptl_xr[i][j][k] = new double*[NXValue];
							for (int h = 0; h < NXValue; h++)
								ptl_xr[i][j][k][h] = new double[2];
						}
					}
				}
				H = new double*[NXValue];
				for (int i = 0; i < NXValue; i++)
					H[i] = new double[2];
				EH = new double[NXValue];
				rec = new Rec[NTrial];

				ipos = 0;

				for (int i = 0; i < Ng; i++)
					for (int j = 0; j < Na; j++)
						for (int k = 0; k < Nb; k++)
							ptlambda0[i][j][k] = ptlambda[i][j][k] = 1.0 / (Ng * Na * Nb);
			}

			double v_Ng( int i ){
				return i / (Ng - 1.0);		//		0 <= i <= Ng - 1
			}
			double v_Na( int i ){
				return La + i * 0.3;
			}
			double v_Nb( int i ){
				return Lb + i * 1.0;
			}
			double x_value( int i ){
				return double(i + 1);			//		i = 1, ..., NXValue;
			}

			void Prepare();
			void CalcPost();
	};  //  Psi


	 extern Psi * psi1;
	 extern Psi * psi2;

	 extern double * Pg, * Pa, * Pb;
	 extern int imgx0, imgy0, imgyH, imgyL;
	 extern int * StSer;
	 extern int itrial;
	 extern Psi * VPsi;

	 using namespace MyLib;

	 extern WHrn * RN;

	 extern myrn * rnk;

}

#endif
