#include "stdafx.h"

//   Version 2008.08.25, 2009.10

#include <math.h>

//#include "mylib.h"
#include "rnkiss.h"

using namespace System;

namespace MyRNKISS {

	inline double sqr( double v ){
			return v * v;
	}

/****************************************************************

                    The Kiss Algorithm

              See   C.P.Robert and G.Casella
             "Monte Carlo Statistical Methods"
                 1999, Springer-Verlag
                         Pp. 39-43.

******************************************************************/


	myrn::myrn() {
		x = 0x95555555;
		y = 0x56666666;
		z = 0x39999999;
	}

	myrn::myrn( unsigned __int32 xv, unsigned __int32 yv, unsigned __int32 zv ){
		x = xv;
		y = yv;
		z = zv;
	}


	void myrn::init(){
		x = 0x95555555;
		y = 0x56666666;
		z = 0x39999999;
	}


	void myrn::init( unsigned __int32 xv, unsigned __int32 yv, unsigned __int32 zv ){
		x = xv;
		y = yv;
		z = zv;
	}


	//
	//     cf. Robert and Casella (1999) "Monte Carlo Statistical Methods", 
	//                                    Algorithm A.3
	//
	//                     0.0 <  uni()  <  1.0      
	//
	double myrn::uni() {
       const double den = ((double)0xFFFFFFFF) + 2.0;

		x = 69069 * x + 23606797;
		y ^= y << 17;
		y ^= y >> 15;
		z = (z ^ (z << 18)) & 0x7FFFFFFF;
		z ^= z >> 13;

		unsigned __int32 v = x + y + z;

		return (((double)v) + 1.0) / den;
	}


    //        Standard Normal Distribution
	//        Rejection polar method for normal variate
	double myrn::normal(){
		double v1, v2, w;
		do {
    		v1 = 2.0 * uni() - 1.0;
			v2 = 2.0 * uni() - 1.0;
			w  = sqr(v1) + sqr(v2);
		} while (((w >= 1.0) || (w <= 0.0)));

		double c = Math::Sqrt(-2.0 * Math::Log(w)/w);

		return c * v1;
	}


    //    Normal Distribution of mean m  and standard deviation s
	double myrn::normalMS(double m, double s){
			double v = s * normal() + m;
			return  v;
	}


    //    A Pair of Independent Random Numbers of 
	//      the Standard Normal Distribution 
	void  myrn::normalPair( double & n1, double & n2 ){
			double v1, v2, w, c;
			do {
				  v1 = 2.0 * uni() - 1.0;
				  v2 = 2.0 * uni() - 1.0;
				  w  = sqr(v1) + sqr(v2);
			} while (((w >= 1.0) || (w <= 0.0)));

			c = Math::Sqrt(-2.0 * Math::Log(w) / w);

			n1 = c * v1;
			n2 = c * v2;
	}


}
